/**************************************************************************
*  This file is part of the TAL project (Tiny Abstraction Layer)
*
*  Copyright (c) 2017-2018 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  17.03.2017  mifi  First Version, tested with an Embedded Artists 
*                    LPC4088 Quickstart Board and QSB4088com.
**************************************************************************/
#if !defined(__TALBOARD_H__) && defined(USE_BOARD_BF4088)
#define __TALBOARD_H__

/**************************************************************************
*  Includes
**************************************************************************/
#include "project.h"
#include "taltypes.h"

#include "chip.h"

/**************************************************************************
*  Global Definitions
**************************************************************************/

/*
 * Defines for board and cpu name
 */
#define TAL_BOARD "BF4088"
#define TAL_CPU   "LPC4088"


/*
 * Check OSCILLATOR_CLOCK_FREQUENCY value, must be 12MHz
 */
#if (OSCILLATOR_CLOCK_FREQUENCY != 12000000)
   Error: OSCILLATOR_CLOCK_FREQUENCY must defined to 12000000 by the preprocessor;
#endif   


/*
 * With 5 bits of interrupt priority 32 levels are
 * available. Priority 0 is the highest one, 31 the lowest.
 */

#ifndef SYSTICK_PRIO
#define SYSTICK_PRIO       0
#endif 

#ifndef UART0_PRIO
#define UART0_PRIO         7
#endif

#ifndef UART1_PRIO
#define UART1_PRIO         7
#endif

#ifndef UART2_PRIO
#define UART2_PRIO         7
#endif

#ifndef UART3_PRIO
#define UART3_PRIO         7
#endif

#ifndef UART4_PRIO
#define UART4_PRIO         7
#endif

#ifndef CAN_PRIO
#define CAN_PRIO           6
#endif


/*
 * The Embedded Artists LPC4088 Developer's Kit support 2 LEDs
 */
typedef enum _tal_led_channel_
{
   TAL_LED_CHANNEL_1 = 0,
   TAL_LED_CHANNEL_2,
   TAL_LED_CHANNEL_3,
   TAL_LED_CHANNEL_4,
   
   /* TAL_LED_CHANNEL_MAX must be the last one */
   TAL_LED_CHANNEL_MAX
} TAL_LED_CHANNEL;

/*
 * LED name from the board itself
 */
#define LED1   TAL_LED_CHANNEL_1
#define LED2   TAL_LED_CHANNEL_2

/*
 * COM port used for the terminal
 */
#if !defined(TERM_COM_PORT)
/*
 * FTDI  = TAL_COM_PORT_4
 * RS232 = TAL_COM_PORT_1
 */
#define TERM_COM_PORT   TAL_COM_PORT_1
#endif

/*
 * USB device port
 */
 
/* Comment out if PORT_1 should be used */
#define USE_USB_PORT_2

/*
 * Configure external memory for the HEAP
 */
#define TAL_HEAP_MEM1_START   __RAM2_segment_used_end__
#define TAL_HEAP_MEM1_END     __RAM2_segment_end__

/**************************************************************************
*  Macro Definitions
**************************************************************************/

/**************************************************************************
*  Functions Definitions
**************************************************************************/

void       tal_BoardMemInit (void);

TAL_RESULT tal_BoardEnableCOM1 (void);
TAL_RESULT tal_BoardEnableCOM2 (void);
TAL_RESULT tal_BoardEnableCOM3 (void);
TAL_RESULT tal_BoardEnableCOM4 (void);
TAL_RESULT tal_BoardEnableCOM5 (void);

TAL_RESULT tal_BoardEnableCAN1 (void);
TAL_RESULT tal_BoardEnableCAN2 (void);

TAL_RESULT tal_BoardGetMACAddress (uint8_t *pAddress);



#define SD_PRESENT         ((uint8_t)0x01)
#define SD_NOT_PRESENT     ((uint8_t)0x00)

uint8_t SD_Detect (void);

extern int32_t NutHeapUsedRawMemory;

#endif /* !__TALBOARD_H__ */

/*** EOF ***/
