/**************************************************************************
*  Copyright (c) 2014-2017 by Michael Fischer.
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  27.07.2014  mifi  First Version, tested with a STM3240G-EVAL board.
*  12.05.2017  mifi  Rework for the Tiny Cooperative Task Scheduler (CTS).
**************************************************************************/
#define __OPENER_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdio.h>
#include <stdint.h>

#include "project.h"
#include "ipstack.h"

#include "opener_api.h"
#include "trace.h"
#include "networkhandler.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/* 
 * Some TASK variables like stack and task control block.
 */
static OS_STACK (MainStack,  TASK_IP_EIP_MAIN_STK_SIZE);

static os_tcb_t TCBMainTask;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  MainTask                                                             */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static void MainTask (void *p)
{
   (void)p;

   while(!IPIsNetIfUp())
   {
      OSTimeDly(500);
   }
   
   NetworkHandler_Init();
   
   
   while(1)
   {
      OSTimeDly(OS_MS_2_TICKS(10));
      
      NetworkHandler_ProcessOnce();
//      manageConnections();
   }

} /* MainTask */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  eip_Start                                                            */
/*                                                                       */
/*  Take a look at "Porting Guide".                                      */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void eip_Start (void)
{
   uint8_t  MyMACAddress[6];
   uint32_t SerialNumber;
   uint16_t UniqueConnID;
   
   OPENER_TRACE_INFO("OpENerSetup\n");
   
   /*
    * 1. Configure the network properties
    */
   configureNetworkInterface(IP_DEFAULT_ADDRESS, 
                             IP_DEFAULT_NETMASK, 
                             IP_DEFAULT_GATEWAY);

   MyMACAddress[0] = IP_MAC_ADDRESS_0;
   MyMACAddress[1] = IP_MAC_ADDRESS_1;
   MyMACAddress[2] = IP_MAC_ADDRESS_2;
   MyMACAddress[3] = IP_MAC_ADDRESS_3;
   MyMACAddress[4] = IP_MAC_ADDRESS_4;
   MyMACAddress[5] = IP_MAC_ADDRESS_5;
   configureMACAddress(MyMACAddress);
   
   configureDomainName("test.com");
   configureHostName("testdevice");
   
   /*
    * 2. Set device's serial number
    */
   SerialNumber = (IP_MAC_ADDRESS_3 << 16) | (IP_MAC_ADDRESS_4 << 8) | IP_MAC_ADDRESS_5;
   setDeviceSerialNumber(SerialNumber);
      
   /*
    * 3. Initialize OpENer
    */
   srand(SerialNumber); 
   UniqueConnID = (uint16_t)rand();
   CIP_Init(UniqueConnID);
   
   /*
    * 4. Create Application Specific CIP Objects
    */
   /* This will be done by the IApp_Init callback. */    
   
   
   /*
    * 5. Setup the listening TCP and UDP port
    */
   /* This will be done from the main task */

   /* Create the EIP task */
   OSTaskCreate(&TCBMainTask, MainTask, NULL, TASK_IP_EIP_MAIN_PRIORITY,
                MainStack, sizeof(MainStack), 
                "EIP");

} /* eip_Start */

/*** EOF ***/


